@@ -156,6 +156,24 @@
 /*	\fBnever\fR (don't send any notifications at all).
 /*
 /*	This feature is available in Postfix 2.3 and later.
+/* .IP "\fB-O requiretls"
+/*	When delivering the message with SMTP, the connection must use TLS
+/*	with a verified server certificate, and the remote SMTP server
+/*	must support REQUIRETLS. Try multiple SMTP servers if possible,
+/*	and return the message as undeliverable when these requirements
+/*	were not satisfied with any of the remote SMTP servers that were
+/*	tried. The "requiretls" option value is case-insensitive.
+/*
+/*	This feature is available in Postfix 3.10 and later.
+/* .IP "\fB-O smtputf8"
+/*	When delivering the message with SMTP, the connection must use
+/*	the SMTPUTF8 extension. Try multiple SMTP servers if possible,
+/*	and return the message as undeliverable when a message contains
+/*	an UTF8 envelope address or message header, but SMTPUTF8 was not
+/*	supported by any of the remote SMTP servers that were tried. The
+/*	"smtputf8" option value is case-insensitive.
+/*
+/*	This feature is available in Postfix 3.10 and later.
 /* .IP "\fB-n\fR (ignored)"
 /*	Backwards compatibility.
 /* .IP "\fB-oA\fIalias_database\fR"
@@ -434,6 +452,11 @@
 /*	the Postfix executable files and documentation with the default
 /*	Postfix instance, and that are started, stopped, etc., together
 /*	with the default Postfix instance.
+/* .PP
+/*	Postfix 3.10 and later:
+/* .IP "\fBrequiretls_enable (yes)\fR"
+/*	Enable support for the ESMTP verb "REQUIRETLS", defined in RFC
+/*	8689.
 /* FILES
 /*	/var/spool/postfix, mail queue
 /*	/etc/postfix, configuration files
@@ -473,8 +496,6 @@
 /*	Google, Inc.
 /*	111 8th Avenue
 /*	New York, NY 10011, USA
-/*
-/*	Wietse Venema
 /*--*/
 
 /* System library. */
@@ -539,6 +560,7 @@
 #include <user_acl.h>
 #include <dsn_mask.h>
 #include <mail_parm_split.h>
+#include <sendopts.h>
 
 /* Application-specific. */
 
@@ -591,6 +613,11 @@ static const CONFIG_STR_TABLE str_table[] = {
     0,
 };
 
+ /*
+  * Sender options.
+  */
+static int sm_sendopts;
+
  /*
   * Silly little macros (SLMs).
   */
@@ -790,6 +817,14 @@ static void enqueue(const int flags, const char *encoding,
      * With "sendmail -N", instead of a per-message NOTIFY record we store one
      * per recipient so that we can simplify the implementation somewhat.
      */
+    if (sm_sendopts)
+	rec_fprintf(dst, REC_TYPE_SIZE, REC_TYPE_SIZE_FORMAT,
+		    (REC_TYPE_SIZE_CAST1) ~ 0,	/* message segment size */
+		    (REC_TYPE_SIZE_CAST2) ~ 0,	/* content offset */
+		    (REC_TYPE_SIZE_CAST3) ~ 0,	/* recipient count */
+		    (REC_TYPE_SIZE_CAST4) ~ 0,	/* qmgr options */
+		    (REC_TYPE_SIZE_CAST5) ~ 0,	/* content length */
+		    (REC_TYPE_SIZE_CAST6) sm_sendopts);
     if (dsn_envid)
 	rec_fprintf(dst, REC_TYPE_ATTR, "%s=%s",
 		    MAIL_ATTR_DSN_ENVID, dsn_envid);
@@ -1253,7 +1288,19 @@ int     main(int argc, char **argv)
 	    break;
 	case 'N':
 	    if ((dsn_notify = dsn_notify_mask(optarg)) == 0)
-		msg_warn("bad -N option value -- ignored");
+		msg_warn("bad -N option value: '%s' -- ignored", optarg);
+	    break;
+	case 'O':
+	    if (strcasecmp(optarg, "REQUIRETLS") == 0) {
+		sm_sendopts |= SOPT_REQUIRETLS_ESMTP;
+	    if (var_requiretls_enable == 0)
+		msg_warn("'-O requiretls' was requested, but the "
+			 "configuration is 'requiretls_enable = no'");
+	    } else if (strcasecmp(optarg, "SMTPUTF8") == 0) {
+		sm_sendopts |= SOPT_SMTPUTF8_REQUESTED;
+	    } else {
+		msg_warn("bad -O option value: '%s' -- ignored", optarg);
+	    }
 	    break;
 	case 'R':
 	    if ((dsn_ret = dsn_ret_code(optarg)) == 0)
@@ -1485,8 +1532,7 @@ int     main(int argc, char **argv)
 	if (alias_map_from_args != 0)
 	    argv_add(ext_argv, alias_map_from_args, (char *) 0);
 	else
-	    argv_split_append_cw(ext_argv, var_alias_db_map, CHARS_COMMA_SP,
-				 VAR_ALIAS_DB_MAP);
+	    argv_split_append(ext_argv, var_alias_db_map, CHARS_COMMA_SP);
 	argv_terminate(ext_argv);
 	mail_run_replace(var_command_dir, ext_argv->argv);
 	/* NOTREACHED */
